// Segment bit values {g, f, e, d, c, b, a}
#define DISPLAY_0 0x3F
#define DISPLAY_1 0x06
#define DISPLAY_2 0x5B
#define DISPLAY_3 0x4F
#define DISPLAY_4 0x66
#define DISPLAY_5 0x6D
#define DISPLAY_6 0x7D
#define DISPLAY_7 0x07
#define DISPLAY_8 0x7F
#define DISPLAY_9 0x6F
#define DISPLAY_C 0x39
#define DISPLAY_A 0x77
#define DISPLAY_P 0x73
#define DISPLAY_BLANK 0x00


int SegPins[] = {LED_SEG_A, LED_SEG_B, LED_SEG_C, LED_SEG_D, LED_SEG_E, LED_SEG_F, LED_SEG_G};

void EvalDisplay() {
  byte CurDigitVal;
  
  if (BlinkDigit == (CurDigit + 1)) {
    if (BlinkCount >= (DIGIT_BLINK_PRD / 2)) {
      CurDigitVal = DISPLAY_BLANK;
    } else {
      CurDigitVal = DigitVals[CurDigit];
    }
    if (BlinkCount == DIGIT_BLINK_PRD) {
      BlinkCount = 0;
    } else {
      BlinkCount++;
    }
  } else {
    CurDigitVal = DigitVals[CurDigit];
  }
  
  switch (CurDigit) {
    case 0: 
      digitalWrite(LED_DIG_3, 0);
      SetSegments(CurDigitVal);
      digitalWrite(LED_DIG_0, 1);
      break;
    case 1:
      digitalWrite(LED_DIG_0, 0);
      SetSegments(CurDigitVal);
      digitalWrite(LED_DIG_1, 1);
      break;
    case 2:
      digitalWrite(LED_DIG_1, 0);
      SetSegments(CurDigitVal);
      digitalWrite(LED_DIG_2, 1);
      break;
    case 3:
      digitalWrite(LED_DIG_2, 0);
      SetSegments(CurDigitVal);
      digitalWrite(LED_DIG_3, 1);
  }
  
  if (CurDigit == 3) {
    CurDigit = 0;
  } else {
    CurDigit++;
  }
}


byte ComputeSegData(unsigned int DigitVal) {
  switch (DigitVal) {
    case 0:
      return DISPLAY_0;
    case 1:
      return DISPLAY_1;
    case 2:
      return DISPLAY_2;
    case 3:
      return DISPLAY_3;
    case 4:
      return DISPLAY_4;
    case 5:
      return DISPLAY_5;
    case 6:
      return DISPLAY_6;
    case 7:
      return DISPLAY_7;
    case 8:
      return DISPLAY_8;
    case 9:
      return DISPLAY_9;
    default:
      return DISPLAY_BLANK;
  }
}


void SetSegments (byte SegData) {
  boolean SegVal;
  for (int i=0; i<8; i++) {
    SegVal = ((~SegData & 0x01) == 0x01);
    SegData = SegData >> 1;
    digitalWrite(SegPins[i], SegVal);
  }
}


void SetupDmxDisplay() {
  unsigned int TrueAddress = dmxaddress - 3;
  DigitVals[0] = ComputeSegData(TrueAddress % 10);
  DigitVals[1] = ComputeSegData((TrueAddress / 10) % 10);
  DigitVals[2] = ComputeSegData(TrueAddress / 100);
  DigitVals[3] = DISPLAY_C;
  BlinkDigit = 0;
}


void SetupSetAddrDisplay(byte d100, byte d10, byte d1) {
  DigitVals[0] = ComputeSegData(d1);
  DigitVals[1] = ComputeSegData(d10);
  DigitVals[2] = ComputeSegData(d100);
  DigitVals[3] = DISPLAY_A;
  BlinkDigit = 3;
  BlinkCount = 0;
}


void SetBlinkDigit(byte digit) {
  // digit 0-2 corresponds with BlinkDigit 1-3
  BlinkDigit = digit+1;
  BlinkCount = 0;
}


void SetAddrDigitDisplay(byte digit, byte val) {
  DigitVals[digit] = ComputeSegData(val);
}


void SetupProg1Display() {
  DigitVals[0] = ComputeSegData(1);
  DigitVals[1] = DISPLAY_BLANK;
  DigitVals[2] = DISPLAY_BLANK;
  DigitVals[3] = DISPLAY_P;
  BlinkDigit = 0;
}


void SetupProg2Display() {
  DigitVals[0] = ComputeSegData(2);
  DigitVals[1] = DISPLAY_BLANK;
  DigitVals[2] = DISPLAY_BLANK;
  DigitVals[3] = DISPLAY_P;
  BlinkDigit = 0;
}


void SetupProg3Display() {
  DigitVals[0] = ComputeSegData(3);
  DigitVals[1] = DISPLAY_BLANK;
  DigitVals[2] = DISPLAY_BLANK;
  DigitVals[3] = DISPLAY_P;
  BlinkDigit = 0;
}


void SetupProg4Display() {
  DigitVals[0] = ComputeSegData(4);
  DigitVals[1] = DISPLAY_BLANK;
  DigitVals[2] = DISPLAY_BLANK;
  DigitVals[3] = DISPLAY_P;
  BlinkDigit = 0;
}
