void InitController() {
  /************************* RS485 IF configuration ******************************/
  pinMode(RECEIVER_OUTPUT_ENABLE, OUTPUT);
  pinMode(DRIVER_OUTPUT_ENABLE, OUTPUT);
  digitalWrite(RECEIVER_OUTPUT_ENABLE, LOW);
  digitalWrite(DRIVER_OUTPUT_ENABLE, LOW);    //sets pins 3 and 4 to low to enable reciever mode on the SN75176.

  pinMode(RX_PIN, INPUT);  //sets serial pin to receive data

  /******************************* PWM Outputs *********************************/
  pinMode(PWM_RED, OUTPUT);
  analogWrite(PWM_RED, 0);
  pinMode(PWM_GREEN,OUTPUT);
  analogWrite(PWM_GREEN, 0);
  pinMode(PWM_BLUE, OUTPUT);
  analogWrite(PWM_BLUE, 0);
  
  /******************************* LED Outputs *********************************/
  pinMode(LED_SEG_A, OUTPUT);  // Segment outputs are driven to 0 to enable the segment
  digitalWrite(LED_SEG_A, 1);
  pinMode(LED_SEG_B, OUTPUT);
  digitalWrite(LED_SEG_B, 1);
  pinMode(LED_SEG_C, OUTPUT);
  digitalWrite(LED_SEG_C, 1);
  pinMode(LED_SEG_D, OUTPUT);
  digitalWrite(LED_SEG_D, 1);
  pinMode(LED_SEG_E, OUTPUT);
  digitalWrite(LED_SEG_E, 1);
  pinMode(LED_SEG_F, OUTPUT);
  digitalWrite(LED_SEG_F, 1);
  pinMode(LED_SEG_G, OUTPUT);
  digitalWrite(LED_SEG_G, 1);
  
  pinMode(LED_DIG_0, OUTPUT);  // Digit outputs are driven to 1 to enable the digit
  digitalWrite(LED_DIG_0, 0);
  pinMode(LED_DIG_1, OUTPUT);
  digitalWrite(LED_DIG_1, 0);
  pinMode(LED_DIG_2, OUTPUT);
  digitalWrite(LED_DIG_2, 0);
  pinMode(LED_DIG_3, OUTPUT);
  digitalWrite(LED_DIG_3, 0);
  
  /***************************** Control Inputs **********************************/
  pinMode(MODE_BUTTON, INPUT);
  
  /**************************** USART configuration ******************************/
  
  Serial.begin(250000);
  /* Each bit is 4uS long, hence 250Kbps baud rate */
  
  cli(); //disable interrupts while we're setting bits in registers
  
  bitClear(UCSR0B, RXCIE0);  //disable USART reception interrupt
  
  /******************************* Timer2 configuration ***********************************/
  
  //NOTE:  this will disable PWM on pins 3 and 11.
  bitClear(TCCR2A, COM2A1);
  bitClear(TCCR2A, COM2A0); //disable compare match output A mode
  bitClear(TCCR2A, COM2B1);
  bitClear(TCCR2A, COM2B0); //disable compare match output B mode
  bitSet(TCCR2A, WGM21);
  bitClear(TCCR2A, WGM20);  //set mode 2, CTC.  TOP will be set by OCRA.
  
  bitClear(TCCR2B, FOC2A);
  bitClear(TCCR2B, FOC2B);  //disable Force Output Compare A and B.
  bitClear(TCCR2B, WGM22);  //set mode 2, CTC.  TOP will be set by OCRA.
  bitClear(TCCR2B, CS22);
  bitClear(TCCR2B, CS21);
  bitSet(TCCR2B, CS20);     //no prescaler means the clock will increment every 62.5ns (assuming 16Mhz clock speed).
  
  OCR2A = 64;                
  /* Set output compare register to 64, so that the Output Compare Interrupt will fire
  *  every 4uS.  */
  
  bitClear(TIMSK2, OCIE2B);  //Disable Timer/Counter2 Output Compare Match B Interrupt
  bitSet(TIMSK2, OCIE2A);    //Enable Timer/Counter2 Output Compare Match A Interrupt
  bitClear(TIMSK2, TOIE2);   //Disable Timer/Counter2 Overflow Interrupt Enable           
      
  sei();                     //reenable interrupts
 
}


void InitVariables() {
  unsigned int IndexColor;
  
  // Get the DMX address from storage
  
  dmxaddress = GetDmxAddress() + 3;
  /*  this will allow the USART receive interrupt to fire an additional 3 times for every dmx frame.  
  *   Here's why:
  *   Once to account for the fact that DMX addresses run from 0-511, whereas channel numbers
  *        start numbering at 1.
  *   Once for the Mark After Break (MAB), which will be detected by the USART as a valid character 
  *        (a zero, eight more zeros, followed by a one)
  *   Once for the START code that precedes the 512 DMX values (used for RDM).  */
  
  // Get the initial operating mode
  UIMode = GetFixtureMode();
  switch (UIMode) {
    case UI_MODE_PROG_1:
      SetupProg1();
      SetupProg1Display();
      break;
      
    case UI_MODE_PROG_2:
      SetupProg2();
      SetupProg2Display();
      // Then set the initial color from storage
      IndexColor = GetFixtureIndexedColor();
      CalcIndexedRGB(IndexColor);
      break;
      
    case UI_MODE_PROG_3:
      SetupProg3();
      SetupProg3Display();
      break;
      
    case UI_MODE_PROG_4:
      SetupProg4();
      SetupProg4Display();
      break;
      
    default:
      SetupDMX();
      SetupDmxDisplay();
  }
   
  PrevTime = micros();
  PrevUIMode = UIMode;
  ModeChangeTimer = 0;
}
